//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#ifndef CAPTURE_FORCE_TORQUE_PREINT_H
#define CAPTURE_FORCE_TORQUE_PREINT_H

//Wolf includes
#include "bodydynamics/math/force_torque_delta_tools.h"
#include "core/capture/capture_base.h"
#include "core/capture/capture_motion.h"
#include "bodydynamics/capture/capture_inertial_kinematics.h"

#include <core/state_block/state_composite.h>

namespace wolf {
    
WOLF_PTR_TYPEDEFS(CaptureForceTorquePreint);

class CaptureForceTorquePreint : public CaptureMotion
{
    public:
        /* 
        capture _data
        name, size: description
        f1  , 3 : force of foot 1 in foot 1 frame
        f2  , 3 : force of foot 2 in foot 2 frame
        tau1, 3 : torque of foot 1 in foot 1 frame
        tau2, 3 : torque of foot 2 in foot 2 frame
        pbc , 3 : position of COM relative to the base in base
        wb  , 3 : angular velocity of body frame in body frame 
        pbl1, 3 : position of foot 1 relative to the base in base frame
        pbl2, 3 : position of foot 2 relative to the base in base frame
        qbl1, 4 : orientation of foot 1 in base frame
        qbl2, 4 : orientation of foot 2 in base frame
        */
        CaptureForceTorquePreint(
                    const TimeStamp& _init_ts,
                    SensorBasePtr _sensor_ptr,
                    CaptureInertialKinematicsPtr _capture_IK_ptr,  // to get the pbc bias
                    CaptureMotionPtr _capture_motion_ptr,          // to get the gyro bias
                    const Eigen::VectorXd& _data,
                    const Eigen::MatrixXd& _data_cov,
                    CaptureBasePtr _capture_origin_ptr = nullptr);
    
        ~CaptureForceTorquePreint() override;

        CaptureBaseConstPtr getIkinCaptureOther() const { return cap_ikin_other_;}
        CaptureBasePtr getIkinCaptureOther(){ return cap_ikin_other_;}
        CaptureBaseConstPtr getGyroCaptureOther() const { return cap_gyro_other_;}
        CaptureBasePtr getGyroCaptureOther(){ return cap_gyro_other_;}

    private:
        CaptureBasePtr cap_ikin_other_;
        CaptureBasePtr cap_gyro_other_;
};

} // namespace wolf

#endif // CAPTURE_FORCE_TORQUE_PREINT_H
