//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#ifndef CAPTURE_INERTIAL_KINEMATICS_H
#define CAPTURE_INERTIAL_KINEMATICS_H

//Wolf includes
#include "bodydynamics/math/force_torque_delta_tools.h"
#include "core/capture/capture_motion.h"
#include "core/capture/capture_base.h"

namespace wolf {
    
WOLF_PTR_TYPEDEFS(CaptureInertialKinematics);

class CaptureInertialKinematics : public CaptureBase
{
    public:

        CaptureInertialKinematics(const TimeStamp& _init_ts,
                                  SensorBasePtr _sensor_ptr,
                                  const Eigen::Vector9d& _data,    // pbc, vbc, wb
                                  const Eigen::Matrix3d & _B_I_q,  // Centroidal inertia matrix expressed in body frame
                                  const Eigen::Vector3d & _B_Lc_m, // Centroidal angular momentum expressed in body frame
                                  const Vector3d& _bias);

        CaptureInertialKinematics(const TimeStamp& _init_ts,
                                  SensorBasePtr _sensor_ptr,
                                  const Eigen::Vector9d& _data,     // pbc, vbc, wb
                                  const Eigen::Matrix3d & _B_I_q,   // Centroidal inertia matrix expressed in body frame
                                  const Eigen::Vector3d & _B_Lc_m); // Centroidal angular momentum expressed in body frame
    
        ~CaptureInertialKinematics() override;

        const Eigen::Vector9d& getData()    {return data_;}
        const Eigen::Matrix3d& getBIq()     {return B_I_q_;}
        const Eigen::Vector3d& getBLcm()    {return B_Lc_m_;}

    private:
        Eigen::Vector9d data_;
        Eigen::Matrix3d B_I_q_;
        Eigen::Vector3d B_Lc_m_;
};
} // namespace wolf

#endif // CAPTURE_INERTIAL_KINEMATICS_H
