//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#ifndef PROCESSOR_INERTIAL_KINEMATICS_H
#define PROCESSOR_INERTIAL_KINEMATICS_H

// Wolf
#include "core/sensor/sensor_base.h"
#include "core/processor/processor_base.h"

#include "core/processor/processor_base.h"
#include "core/factor/factor_base.h"
#include "core/factor/factor_block_difference.h"
#include "bodydynamics/sensor/sensor_inertial_kinematics.h"
#include "bodydynamics/capture/capture_inertial_kinematics.h"
#include "bodydynamics/feature/feature_inertial_kinematics.h"
#include "bodydynamics/factor/factor_inertial_kinematics.h"

namespace wolf {
WOLF_STRUCT_PTR_TYPEDEFS(ParamsProcessorInertialKinematics);

struct ParamsProcessorInertialKinematics : public ParamsProcessorBase
{
    std::string sensor_angvel_name;
    double std_bp_drift;

    ParamsProcessorInertialKinematics() = default;
    ParamsProcessorInertialKinematics(std::string _unique_name, const ParamsServer& _server):
        ParamsProcessorBase(_unique_name, _server)
    {
        sensor_angvel_name = _server.getParam<std::string>(_unique_name + "/sensor_angvel_name");
        std_bp_drift =       _server.getParam<double>(_unique_name + "/std_bp_drift");
    }
    ~ParamsProcessorInertialKinematics() override = default;
    std::string print() const override
    {
        return ParamsProcessorBase::print() + "\n"
                    + "sensor_angvel_name: "       + sensor_angvel_name            + "\n"
                    + "std_bp_drift: "            + std::to_string(std_bp_drift) + "\n";
    }
};

WOLF_PTR_TYPEDEFS(ProcessorInertialKinematics);
    
//class
class ProcessorInertialKinematics : public ProcessorBase{
    public:
        ProcessorInertialKinematics(ParamsProcessorInertialKinematicsPtr _params_ikin);
        ~ProcessorInertialKinematics() override = default;
        WOLF_PROCESSOR_CREATE(ProcessorInertialKinematics, ParamsProcessorInertialKinematics);

        void configure(SensorBasePtr _sensor) override;

        bool createInertialKinematicsFactor(CaptureInertialKinematicsPtr _cap_ikin, CaptureBasePtr _cap_angvel, CaptureBasePtr _cap_ikin_origin);

        void processCapture(CaptureBasePtr _incoming) override;
        void processKeyFrame(FrameBasePtr _keyframe_ptr) override;
        bool triggerInCapture(CaptureBasePtr) const override;
        bool triggerInKeyFrame(FrameBasePtr _keyframe_ptr) const override;
        bool storeKeyFrame(FrameBasePtr) override;
        bool storeCapture(CaptureBasePtr) override;
        bool voteForKeyFrame() const override;


    protected:
        ParamsProcessorInertialKinematicsPtr params_ikin_;
        CaptureBasePtr cap_origin_ptr_;
};



inline bool ProcessorInertialKinematics::triggerInKeyFrame(FrameBasePtr _keyframe_ptr) const
{
    return false;
}

inline bool ProcessorInertialKinematics::triggerInCapture(CaptureBasePtr _capture) const
{   
    return true;
}

inline bool ProcessorInertialKinematics::storeKeyFrame(FrameBasePtr)
{
    return true;
}

inline bool ProcessorInertialKinematics::storeCapture(CaptureBasePtr)
{
    return true;
}

inline bool ProcessorInertialKinematics::voteForKeyFrame() const
{
    return false;
}

inline void ProcessorInertialKinematics::processKeyFrame(FrameBasePtr _keyframe_ptr)
{
}


//////////////////////////
// Covariance computations
//////////////////////////


Eigen::Matrix9d computeIKinJac(const Eigen::Vector3d& w_unb,
                              const Eigen::Vector3d& p_unb,
                              const Eigen::Matrix3d& Iq);

Eigen::Matrix9d computeIKinCov(const Eigen::Matrix3d& Qp,
                              const Eigen::Matrix3d& Qv, 
                              const Eigen::Matrix3d& Qw,
                              const Eigen::Vector3d& p_unb,
                              const Eigen::Vector3d& w_unb,
                              const Eigen::Matrix3d& Iq);

} /* namespace wolf */

/////////////////////////////////////////////////////////
// IMPLEMENTATION. Put your implementation includes here
/////////////////////////////////////////////////////////


namespace wolf{

} // namespace wolf

#endif // PROCESSOR_INERTIAL_KINEMATICS_H
