//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#include "bodydynamics/feature/feature_force_torque.h"

namespace wolf {
    
WOLF_PTR_TYPEDEFS(FeatureForceTorque);

FeatureForceTorque::FeatureForceTorque(
            const double& _dt,
            const double& _mass,
            const Eigen::Vector6d& _forces_meas,
            const Eigen::Vector6d& _torques_meas,
            const Eigen::Vector3d& _pbc_meas,
            const Eigen::Matrix<double,14,1>& _kin_meas,
            const Eigen::Matrix6d& _cov_forces_meas,
            const Eigen::Matrix6d& _cov_torques_meas,
            const Eigen::Matrix3d& _cov_pbc_meas,
            const Eigen::Matrix<double,14,14>& _cov_kin_meas) :
        FeatureBase("FORCETORQUE", 42*Eigen::Matrix1d::Identity(), 42*Eigen::Matrix1d::Identity(), UNCERTAINTY_IS_COVARIANCE),  // Pass dummmy values -> we are bypassing the way meas and cov is usually stored because too many of them == vector is too big -> error prone
        dt_(_dt),
        mass_(_mass),
        forces_meas_(_forces_meas),
        torques_meas_(_torques_meas),
        pbc_meas_(_pbc_meas),
        kin_meas_(_kin_meas),
        cov_forces_meas_(_cov_forces_meas),
        cov_torques_meas_(_cov_torques_meas),
        cov_pbc_meas_(_cov_pbc_meas),
        cov_kin_meas_(_cov_kin_meas)
{}

FeatureForceTorque::~FeatureForceTorque(){}

} // namespace wolf
