//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------

#include "bodydynamics/sensor/sensor_force_torque_inertial.h"

#include <core/state_block/factory_state_block.h>
#include <core/state_block/state_block_derived.h>

namespace wolf
{

// TODO: remove this constructor after merging MR !448
SensorForceTorqueInertial::SensorForceTorqueInertial(const VectorXd&                    _extrinsics,
                                                     ParamsSensorForceTorqueInertialPtr _params)
    : SensorBase("SensorForceTorqueInertial",
                 FactoryStateBlock::create("StatePoint3d", _extrinsics.head(3), true),
                 FactoryStateBlock::create("StateQuaternion", _extrinsics.tail(4), true),
                 FactoryStateBlock::create("StateParams6", Vector6d::Zero(), false),  // IMU bias
                 12,
                 false,
                 false,
                 false),
      params_fti_(_params)
{
    addStateBlock('C', FactoryStateBlock::create("StateParams3", params_fti_->com, true));  // centre of mass
    addStateBlock('i', FactoryStateBlock::create("StateParams3", params_fti_->inertia, true));  // inertial vector
    addStateBlock('m', FactoryStateBlock::create("StateParams1", Vector1d(params_fti_->mass), true));  // total mass
    setStateBlockDynamic('I', false);
    setStateBlockDynamic('C', false);
    setStateBlockDynamic('i', false);
    setStateBlockDynamic('m', false);

}

// TODO: Adapt this API to that of MR !448
SensorForceTorqueInertial::SensorForceTorqueInertial(const VectorComposite&             _states,
                                                     ParamsSensorForceTorqueInertialPtr _params)
    : SensorBase("SensorForceTorqueInertial",
                 FactoryStateBlock::create("StatePoint3d", _states.at('P'), false),
                 FactoryStateBlock::create("StateQuaternion", _states.at('O'), false),
                 FactoryStateBlock::create("StateParams6", _states.at('I'), false),  // IMU bias
                 12,
                 false,
                 false,
                 false),
      params_fti_(_params)
{
    auto sbc = emplaceStateBlock<StateParams3>('C', getProblem(), _states.at('C'), true);  // centre of mass
    auto sbi = emplaceStateBlock<StateParams3>('i', getProblem(), _states.at('i'), true);  // inertial vector
    auto sbm = emplaceStateBlock<StateParams1>('m', getProblem(), _states.at('m'), true);  // total mass
    setStateBlockDynamic('I', false);
    setStateBlockDynamic('C', false);
    setStateBlockDynamic('i', false);
    setStateBlockDynamic('m', false);
}

}  // namespace wolf

#include <core/sensor/factory_sensor.h>
namespace wolf{
WOLF_REGISTER_SENSOR(SensorForceTorqueInertial);
WOLF_REGISTER_SENSOR_AUTO(SensorForceTorqueInertial);
}