%%
function [acc, J_acc_force, J_acc_torque, J_acc_angvel, J_acc_com, J_acc_inertia, J_acc_mass] = forces2acc(force, torque, angvel, com, inertia, mass)

I       = diag(inertia);

angacc  = inv(I) * (torque - cross(angvel, I * angvel));

acc     = force / mass - cross(angacc, com) - cross(angvel, cross(angvel, com));

% Jacobians

fx = force(1);
fy = force(2);
fz = force(3);
tx = torque(1);
ty = torque(2);
tz = torque(3);
wx = angvel(1);
wy = angvel(2);
wz = angvel(3);
cx = com(1);
cy = com(2);
cz = com(3);
ix = inertia(1);
iy = inertia(2);
iz = inertia(3);

J_acc_force = [...
    [1/mass,      0,      0]
    [     0, 1/mass,      0]
    [     0,      0, 1/mass]];

J_acc_torque = [... 
[     0, -cz/iy,  cy/iz]
[ cz/ix,      0, -cx/iz]
[-cy/ix,  cx/iy,      0]];
 
 
J_acc_angvel = [...
[-((cy*iy*wy + cz*iz*wz)*(iy - ix + iz))/(iy*iz),       2*cx*wy - cy*wx + (cy*(ix*wx - iy*wx))/iz,       2*cx*wz - cz*wx + (cz*(ix*wx - iz*wx))/iy]
[      2*cy*wx - cx*wy - (cx*(ix*wy - iy*wy))/iz, -((cx*ix*wx + cz*iz*wz)*(ix - iy + iz))/(ix*iz),       2*cy*wz - cz*wy + (cz*(iy*wy - iz*wy))/ix]
[      2*cz*wx - cx*wz - (cx*(ix*wz - iz*wz))/iy,       2*cz*wy - cy*wz - (cy*(iy*wz - iz*wz))/ix, -((cx*ix*wx + cy*iy*wy)*(ix + iy - iz))/(ix*iy)]];

 
J_acc_com = [...
[                            wy^2 + wz^2,   (tz + ix*wx*wy - iy*wx*wy)/iz - wx*wy, - wx*wz - (ty - ix*wx*wz + iz*wx*wz)/iy]
[- wx*wy - (tz + ix*wx*wy - iy*wx*wy)/iz,                             wx^2 + wz^2,   (tx + iy*wy*wz - iz*wy*wz)/ix - wy*wz]
[  (ty - ix*wx*wz + iz*wx*wz)/iy - wx*wz, - wy*wz - (tx + iy*wy*wz - iz*wy*wz)/ix,                             wx^2 + wy^2]];
 
 
J_acc_inertia = [... 
[                         (cy*wx*wy)/iz + (cz*wx*wz)/iy,   (cz*(ty - ix*wx*wz + iz*wx*wz))/iy^2 - (cy*wx*wy)/iz, - (cy*(tz + ix*wx*wy - iy*wx*wy))/iz^2 - (cz*wx*wz)/iy]
[- (cz*(tx + iy*wy*wz - iz*wy*wz))/ix^2 - (cx*wx*wy)/iz,                          (cx*wx*wy)/iz + (cz*wy*wz)/ix,   (cx*(tz + ix*wx*wy - iy*wx*wy))/iz^2 - (cz*wy*wz)/ix]
[  (cy*(tx + iy*wy*wz - iz*wy*wz))/ix^2 - (cx*wx*wz)/iy, - (cx*(ty - ix*wx*wz + iz*wx*wz))/iy^2 - (cy*wy*wz)/ix,                          (cx*wx*wz)/iy + (cy*wy*wz)/ix]];
 
 
J_acc_mass = [... 
-fx/mass^2
-fy/mass^2
-fz/mass^2];
 

end

%%
function f()
%%
syms fx fy fz tx ty tz wx wy wz cx cy cz ix iy iz mass real;

force = [fx;fy;fz];
torque = [tx;ty;tz];
angvel = [wx;wy;wz];
com = [cx;cy;cz];
inertia = [ix;iy;iz];

[acc, J_acc_force, J_acc_torque, J_acc_angvel, J_acc_com, J_acc_inertia, J_acc_mass] = forces2acc(force, torque, angvel, com, inertia, mass);

J_acc_force_     = simplify(jacobian(acc, force));
J_acc_torque_    = simplify(jacobian(acc, torque));
J_acc_angvel_    = simplify(jacobian(acc, angvel));
J_acc_com_       = simplify(jacobian(acc, com));
J_acc_inertia_   = simplify(jacobian(acc, inertia));
J_acc_mass_      = simplify(jacobian(acc, mass));

simplify(J_acc_force_ - J_acc_force)
simplify(J_acc_torque_ - J_acc_torque)
simplify(J_acc_angvel_ - J_acc_angvel)
simplify(J_acc_com_ - J_acc_com)
simplify(J_acc_inertia_ - J_acc_inertia)
simplify(J_acc_mass_ - J_acc_mass)


end

function g()
%%
force = [1;2;3];
torque = [4;5;6];
angvel = [3;2;1];
com = [6;5;4];
inertia = [1;-2;3];
mass = 2;

[acc, J_acc_force, J_acc_torque, J_acc_angvel, J_acc_com, J_acc_inertia, J_acc_mass] = forces2acc(force, torque, angvel, com, inertia, mass);

acc
J_acc_force
J_acc_torque
J_acc_angvel
J_acc_com
J_acc_inertia
J_acc_mass

%%
end